@@ -1,6 +1,7 @@
 package controller
 
 import (
+	"fmt"
 	"reflect"
 	"strings"
 	"testing"
@@ -30,6 +31,8 @@ import (
 	"kubevirt.io/containerized-data-importer/pkg/util/cert"
 	"kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/api"
 	sdkapi "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/api"
+
+	ocpconfigv1 "github.com/openshift/api/config/v1"
 )
 
 var (
@@ -175,6 +178,8 @@ func createClient(objs ...runtime.Object) client.Client {
 	// Register cdi types with the runtime scheme.
 	s := scheme.Scheme
 	cdiv1.AddToScheme(s)
+	// Register other types with the runtime scheme.
+	ocpconfigv1.AddToScheme(s)
 	// Create a fake client to mock API calls.
 	return fake.NewFakeClientWithScheme(s, objs...)
 }
@@ -253,6 +258,25 @@ var _ = Describe("setConditionFromPod", func() {
 		Expect(result[AnnRunningConditionMessage]).To(Equal("container is waiting"))
 		Expect(result[AnnRunningConditionReason]).To(Equal("Pending"))
 	})
+
+	It("Should set preallocation status", func() {
+		result := make(map[string]string)
+		testPod := createImporterTestPod(createPvc("test", metav1.NamespaceDefault, nil, nil), "test", nil)
+		testPod.Status = v1.PodStatus{
+			ContainerStatuses: []v1.ContainerStatus{
+				{
+					State: v1.ContainerState{
+						Terminated: &v1.ContainerStateTerminated{
+							Message: "container completed, " + common.PreallocationApplied,
+							Reason:  "Completed",
+						},
+					},
+				},
+			},
+		}
+		setConditionFromPodWithPrefix(result, AnnRunningCondition, testPod)
+		Expect(result[AnnPreallocationApplied]).To(Equal("true"))
+	})
 })
 
 var _ = Describe("GetPreallocation", func() {
@@ -314,34 +338,83 @@ var _ = Describe("GetDefaultStorageClass", func() {
 	})
 })
 
-var _ = Describe("GetStorageClassNameForDV", func() {
-	It("Should return DV's StorageClass if defined", func() {
+var _ = Describe("GetClusterWideProxy", func() {
+	var proxyHTTPURL = "http://user:pswd@www.myproxy.com"
+	var proxyHTTPSURL = "https://user:pswd@www.myproxy.com"
+	var noProxyDomains = ".noproxy.com"
+	var trustedCAName = "user-ca-bundle"
+
+	It("Should return a not empty cluster wide proxy obj", func() {
+		client := createClient(createClusterWideProxy(proxyHTTPURL, proxyHTTPSURL, noProxyDomains, trustedCAName))
+		proxy, err := GetClusterWideProxy(client)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(proxy).ToNot(BeNil())
+
+		By("should return a proxy https url")
+		Expect(proxyHTTPSURL).To(Equal(proxy.Status.HTTPSProxy))
+
+		By("should return a proxy http url")
+		Expect(proxyHTTPURL).To(Equal(proxy.Status.HTTPProxy))
+
+		By("should return a noProxy list of domains")
+		Expect(noProxyDomains).To(Equal(proxy.Status.NoProxy))
+
+		By("should return a CA ConfigMap name")
+		Expect(trustedCAName).To(Equal(proxy.Spec.TrustedCA.Name))
+	})
+
+	It("Should return a nil cluster wide proxy obj", func() {
 		client := createClient()
-		dv := createDataVolumeWithStorageClass("test-name", "test-ns", "test-storage-class")
-		scName := GetStorageClassNameForDV(client, dv)
-		Expect(scName).To(Equal("test-storage-class"))
+		proxy, err := GetClusterWideProxy(client)
+		Expect(err).To(HaveOccurred())
+		Expect(proxy).To(BeNil())
+	})
+})
+
+var _ = Describe("GetImportProxyConfig", func() {
+	var proxyHTTPURL = "http://user:pswd@www.myproxy.com"
+	var proxyHTTPSURL = "https://user:pswd@www.myproxy.com"
+	var noProxyDomains = ".noproxy.com"
+	var trustedCAName = "user-ca-bundle"
+
+	It("should return valid proxy information from a CDIConfig with importer proxy configured", func() {
+		cdiConfig := MakeEmptyCDIConfigSpec("cdiconfig")
+		cdiConfig.Status.ImportProxy = createImportProxy(proxyHTTPURL, proxyHTTPSURL, noProxyDomains, trustedCAName)
+		field, _ := GetImportProxyConfig(cdiConfig, common.ImportProxyHTTP)
+		Expect(proxyHTTPURL).To(Equal(field))
+		field, _ = GetImportProxyConfig(cdiConfig, common.ImportProxyHTTPS)
+		Expect(proxyHTTPSURL).To(Equal(field))
+		field, _ = GetImportProxyConfig(cdiConfig, common.ImportProxyNoProxy)
+		Expect(noProxyDomains).To(Equal(field))
+		field, _ = GetImportProxyConfig(cdiConfig, common.ImportProxyConfigMapName)
+		Expect(trustedCAName).To(Equal(field))
 	})
 
-	It("Should return default StorageClass if not specified in DV or VM", func() {
-		client := createClient(
-			createStorageClass("test-storage-class-1", nil),
-			createStorageClass("test-storage-class-2", map[string]string{
-				AnnDefaultStorageClass: "true",
-			}),
-		)
-		dv := createDataVolume("test-name", "test-ns")
-		scName := GetStorageClassNameForDV(client, dv)
-		Expect(scName).To(Equal("test-storage-class-2"))
+	It("should return blank proxy information from a CDIConfig with importer proxy not configured", func() {
+		cdiConfig := MakeEmptyCDIConfigSpec("cdiconfig")
+		cdiConfig.Status.ImportProxy = createImportProxy("", "", "", "")
+		field, _ := GetImportProxyConfig(cdiConfig, common.ImportProxyHTTP)
+		Expect("").To(Equal(field))
+		field, _ = GetImportProxyConfig(cdiConfig, common.ImportProxyHTTPS)
+		Expect("").To(Equal(field))
+		field, _ = GetImportProxyConfig(cdiConfig, common.ImportProxyNoProxy)
+		Expect("").To(Equal(field))
+		field, _ = GetImportProxyConfig(cdiConfig, common.ImportProxyConfigMapName)
+		Expect("").To(Equal(field))
 	})
 
-	It("Should return empty StorageClass if not defined anywhere", func() {
-		client := createClient(
-			createStorageClass("test-storage-class-1", nil),
-			createStorageClass("test-storage-class-2", nil),
-		)
-		dv := createDataVolume("test-name", "test-ns")
-		scName := GetStorageClassNameForDV(client, dv)
-		Expect(scName).To(Equal(""))
+	It("should return error if the requested field does not exist", func() {
+		cdiConfig := MakeEmptyCDIConfigSpec("cdiconfig")
+		cdiConfig.Status.ImportProxy = createImportProxy("", "", "", "")
+		_, err := GetImportProxyConfig(cdiConfig, "nonExistingField")
+		Expect(err.Error()).To(ContainSubstring(fmt.Sprintf("CDIConfig ImportProxy does not have the field: %s\n", "nonExistingField")))
+	})
+
+	It("should return error if the ImportProxy field is nil", func() {
+		cdiConfig := MakeEmptyCDIConfigSpec("cdiconfig")
+		cdiConfig.Status.ImportProxy = nil
+		_, err := GetImportProxyConfig(cdiConfig, common.ImportProxyHTTP)
+		Expect(err.Error()).To(ContainSubstring(fmt.Sprintf("failed to get field, the CDIConfig ImportProxy is nil\n")))
 	})
 })
 
@@ -427,7 +500,7 @@ func createPendingPvc(name, ns string, annotations, labels map[string]string) *v
 }
 
 func createPvcInStorageClass(name, ns string, storageClassName *string, annotations, labels map[string]string, phase v1.PersistentVolumeClaimPhase) *v1.PersistentVolumeClaim {
-	return &v1.PersistentVolumeClaim{
+	pvc := &v1.PersistentVolumeClaim{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:        name,
 			Namespace:   ns,
@@ -448,6 +521,8 @@ func createPvcInStorageClass(name, ns string, storageClassName *string, annotati
 			Phase: phase,
 		},
 	}
+	pvc.Status.Capacity = pvc.Spec.Resources.Requests.DeepCopy()
+	return pvc
 }
 
 func createScratchPvc(pvc *v1.PersistentVolumeClaim, pod *v1.Pod, storageClassName string) *v1.PersistentVolumeClaim {
@@ -578,6 +653,22 @@ func createStorageClass(name string, annotations map[string]string) *storagev1.S
 	}
 }
 
+func createStorageProfile(name string, accessModes []v1.PersistentVolumeAccessMode, volumeMode v1.PersistentVolumeMode) *cdiv1.StorageProfile {
+	return &cdiv1.StorageProfile{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: name,
+		},
+		Status: cdiv1.StorageProfileStatus{
+			StorageClass: &name,
+			ClaimPropertySets: []cdiv1.ClaimPropertySet{
+				{
+					AccessModes: accessModes,
+					VolumeMode:  &volumeMode,
+				}},
+		},
+	}
+}
+
 func createStorageClassWithBindingMode(name string, annotations map[string]string, bindingMode storagev1.VolumeBindingMode) *storagev1.StorageClass {
 	return &storagev1.StorageClass{
 		VolumeBindingMode: &bindingMode,
@@ -692,34 +783,14 @@ func createVolumeSnapshotCrd() *extv1.CustomResourceDefinition {
 	}
 }
 
-func createDefaultPodResourceRequirements(limitCPUValue string, limitMemoryValue string, requestCPUValue string, requestMemoryValue string) *corev1.ResourceRequirements {
-	if limitCPUValue == "" {
-		limitCPUValue = defaultCPULimit
-	}
-	cpuLimit, err := resource.ParseQuantity(limitCPUValue)
-	Expect(err).ToNot(HaveOccurred())
-	if limitMemoryValue == "" {
-		limitMemoryValue = defaultMemLimit
-	}
-	memLimit, err := resource.ParseQuantity(limitMemoryValue)
-	Expect(err).ToNot(HaveOccurred())
-	if requestCPUValue == "" {
-		requestCPUValue = defaultCPURequest
-	}
-	cpuRequest, err := resource.ParseQuantity(requestCPUValue)
-	Expect(err).ToNot(HaveOccurred())
-	if requestMemoryValue == "" {
-		requestMemoryValue = defaultMemRequest
-	}
-	memRequest, err := resource.ParseQuantity(requestMemoryValue)
-	Expect(err).ToNot(HaveOccurred())
+func createDefaultPodResourceRequirements(limitCPUValue int64, limitMemoryValue int64, requestCPUValue int64, requestMemoryValue int64) *corev1.ResourceRequirements {
 	return &corev1.ResourceRequirements{
 		Limits: map[corev1.ResourceName]resource.Quantity{
-			corev1.ResourceCPU:    cpuLimit,
-			corev1.ResourceMemory: memLimit},
+			corev1.ResourceCPU:    *resource.NewQuantity(limitCPUValue, resource.DecimalSI),
+			corev1.ResourceMemory: *resource.NewQuantity(limitMemoryValue, resource.DecimalSI)},
 		Requests: map[corev1.ResourceName]resource.Quantity{
-			corev1.ResourceCPU:    cpuRequest,
-			corev1.ResourceMemory: memRequest},
+			corev1.ResourceCPU:    *resource.NewQuantity(requestCPUValue, resource.DecimalSI),
+			corev1.ResourceMemory: *resource.NewQuantity(requestMemoryValue, resource.DecimalSI)},
 	}
 }
 
@@ -760,3 +831,38 @@ func createCDIWithWorkload(name, uid string) *cdiv1.CDI {
 		},
 	}
 }
+
+func createClusterWideProxy(HTTPProxy string, HTTPSProxy string, noProxy string, trustedCAName string) *ocpconfigv1.Proxy {
+	proxy := &ocpconfigv1.Proxy{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: ClusterWideProxyName,
+			UID:  types.UID(ClusterWideProxyAPIKind + "-" + ClusterWideProxyName),
+		},
+		Spec: ocpconfigv1.ProxySpec{
+			HTTPProxy:          HTTPProxy,
+			HTTPSProxy:         HTTPSProxy,
+			NoProxy:            noProxy,
+			ReadinessEndpoints: []string{},
+			TrustedCA: ocpconfigv1.ConfigMapNameReference{
+				Name: trustedCAName,
+			},
+		},
+		Status: ocpconfigv1.ProxyStatus{
+			HTTPProxy:  HTTPProxy,
+			HTTPSProxy: HTTPSProxy,
+			NoProxy:    noProxy,
+		},
+	}
+	return proxy
+}
+
+func createClusterWideProxyCAConfigMap(certBytes string) *corev1.ConfigMap {
+	configMap := &v1.ConfigMap{
+		TypeMeta:   metav1.TypeMeta{},
+		ObjectMeta: metav1.ObjectMeta{Name: ClusterWideProxyConfigMapName, Namespace: ClusterWideProxyConfigMapNameSpace},
+		Immutable:  new(bool),
+		Data:       map[string]string{ClusterWideProxyConfigMapKey: string(certBytes)},
+		BinaryData: map[string][]byte{},
+	}
+	return configMap
+}
